%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Opdracht 3 NTV1 - Informatie-extractie %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Predikaat voor het extraheren van feiten. Slaagt meerdere malen
%% als er meerdere feiten zijn.
extract_facts(Result,Fact) :-
    alpino_data:result(Result,[Cat],_),
    alpino_data:dt(Cat,Dt),
    simplify_dt_ntv(Dt,SDt),
    fact(SDt,Fact).

%% Feiten die we kunnen vinden in een subject-predicaat structuur.
fact(Dt,Fact) :-
    Dt:hwrd <=> ben,
    su_pred_fact(Dt,Fact).

%% Amsterdam is de hoofdstad van Nederland
%% Amsterdam is de grootste stad en hoofdstad van Nederland
su_pred_fact(Dt,capital(Country,City)) :-
    subject_word(Dt,City),
    predicate_word(Dt,hoofdstad,PredDt),
    predicate_modifier_dt(PredDt,van,prep,ModDt),
    ModDt:obj1:hwrd <=> Country.

%% Woord dat het onderwerp is.
subject_word(Dt,Word) :-
    Dt:su:hwrd <=> Word.

%% Predicatief complement, waarbij het hoofd Word is. Als er een
%% conjunctie is, zoeken we leden die hieraan voldoen.
predicate_word(Dt,Word,PredDt) :-
    PredDt => dt,
    PredDt:hwrd <=> Word,
    Dt:predc <=> Predc,
    Predc:cnj <=> Cnj,
    predc_single_or_conj(Cnj,Predc,PredDt).

%% Respectievelijk geen of wel een conjunctie van predc's.
predc_single_or_conj([],Predc,Predc) :- !.
predc_single_or_conj(Cnj,_,PredDt) :-
    lists:member(PredDt,Cnj).

%% Modifier met Word als hoofd en Frame als POS-tag, MDt unificeert
%% met deze modifier.
predicate_modifier_dt(Dt,Word,Frame,MDt) :-
    Dt:mod <=> Mods,
    lists:member(MDt,Mods),
    MDt:hwrd <=> Word,
    MDt:frame <=> Frame.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Vanaf hier hoeft er niets veranderd te worden! %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%
%% Hdrug commands %%
%%%%%%%%%%%%%%%%%%%%

:- multifile hdrug_command/3.

hdrug_command(sdt,show_dt_ntv(1),[]).
hdrug_command(sdt,show_dt_ntv(N),[N]).

hdrug_command(facts,format_facts_n(1),[]).
hdrug_command(facts,format_facts_n(N),[N]).

sdt :-
    show_dt_ntv(1).

sdt(N) :-
    show_dt_ntv(N).

facts :-
    format_facts_n(1).

facts(N) :-
    format_facts_n(N).

show_dt_ntv(N) :-
    user:object(N,o(Result,_,_)),
    alpino_data:result(Result,[Cat],_),
    alpino_data:dt(Cat,Dt),
    simplify_dt_ntv(Dt,SDt),
    show(fs,clig,[value(SDt)]).

format_facts_n(N) :-
    object(N,o(Result,_,_)),
    format_facts(Result).

format_facts(Result) :-
    findall(Fact,extract_facts(Result,Fact),Facts),
    format_facts_aux(Facts).

format_facts_aux([]).
format_facts_aux([H|T]) :-
    format("~q.~n",[H]),
    format_facts_aux(T).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% dt feature structure simplification %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

simplify_dt_ntv([],[]) :- !.
simplify_dt_ntv(Dt0,Dt) :-
    Dt => dt,
    simplify_lex(Dt0,Dt),
    alpino_data:dt_daughters(Dt0,SingleValued0,ListValued0),
    simplify_single_valued_daughters(SingleValued0,SingleValued),
    simplify_list_valued_daughters(ListValued0,ListValued),
    unify_daughters(SingleValued,Dt),
    unify_daughters(ListValued,Dt).

simplify_dt_ntv_list([],[]).
simplify_dt_ntv_list([H|T],[NewH|NewT]) :-
    simplify_dt_ntv(H,NewH),
    simplify_dt_ntv_list(T,NewT).

simplify_lex(Dt0,Dt) :-
    (  Dt0:hwrd <=> []
    -> true
    ;  Dt0:hwrd:lexical <=> Dt:hwrd,
       Dt0:frame <=> Frame,
       postag_of_frame(Frame,PosTag),
       Dt:frame <=> PosTag
    ).

simplify_single_valued_daughters([],[]).
simplify_single_valued_daughters([Att/Arg|T],[Att/Arg1|NewT]) :-
    simplify_dt_ntv(Arg,Arg1),
    simplify_single_valued_daughters(T,NewT).

simplify_list_valued_daughters([],[]).
simplify_list_valued_daughters([Att/Arg|T],[Att/Arg1|NewT]) :-
    simplify_dt_ntv_list(Arg,Arg1),
    simplify_list_valued_daughters(T,NewT).

unify_daughters([],_Dt).
unify_daughters([Att/Arg|T],Dt) :-
    Dt:Att <=> Arg,
    unify_daughters(T,Dt).
